/* This program computes asymptotic prediction error loss
** for the DGP
*/
new;
cls;
#include c:\projects\active\mismodel\gauss\mcspec.g

load path = ^mcoutpath Fdgp, Adgp, SIGeps, Wloss;

/* Initialization
*/
RB_mle = zeros(h_n,1);
RV_mle = zeros(h_n,1);

RB_lfe = zeros(h_n,1);
RV_lfe = zeros(h_n,1);

RBall_mle = zeros(h_n,alp_n*p_n);
RBall_lfe = zeros(h_n,alp_n*p_n);

RVall_mle = zeros(h_n,alp_n*p_n);
RVall_lfe = zeros(h_n,alp_n*p_n);

p_ind = 1;
do until p_ind > p_n;
   /* Loop: for each lag-length
   */
   
   p_v = p[p_ind];
   gosub riskcalc;
   
   colind = p_n*seqa(0,1,alp_n)' + p_ind;
   
   RVall_mle[.,colind]
     = ones(1,alp_n).*RV_mle;
   RBall_mle[.,colind]
     = (alp^2)'.*RB_mle;

   RVall_lfe[.,colind]
     = ones(1,alp_n).*RV_lfe;
   RBall_lfe[.,colind]
     = (alp^2)'.*RB_lfe;   

   p_ind = p_ind + 1;
   
endo;

RISK_mle  = RBall_mle + RVall_mle;
RISK_lfe  = RBall_lfe + RVall_lfe;

save path = ^mcoutpath risk_mle, risk_lfe, RBall_mle, RVall_mle, RBall_lfe, RVall_lfe;
clear risk_mle, risk_lfe;

"Press key to print asymptotic results";
kw = keyw;
cls;

/* Screen Output 
*/
format /mat /on /mb1 /rds 10,6;
load path = ^mcoutpath risk_mle, risk_lfe, RBall_mle, RVall_mle, RBall_lfe, RVall_lfe;

h_ind = 1;
do until h_ind > h_n;

   alp_ind = 1;
   do until alp_ind > alp_n;

      RISK_mleprt =  RISK_mle[h_ind,(alp_ind-1)*p_n+1:alp_ind*p_n]  |
                     RBall_mle[h_ind,(alp_ind-1)*p_n+1:alp_ind*p_n] |
                     RVall_mle[h_ind,(alp_ind-1)*p_n+1:alp_ind*p_n] ;

      RISK_lfeprt =  RISK_lfe[h_ind,(alp_ind-1)*p_n+1:alp_ind*p_n]  |
                     RBall_lfe[h_ind,(alp_ind-1)*p_n+1:alp_ind*p_n] |
                     RVall_lfe[h_ind,(alp_ind-1)*p_n+1:alp_ind*p_n] ;

      "Forecast Horizon:" h[h_ind];
      "Misspecification:" alp[alp_ind];
      "-------------------------------------";
      "Maximum Likelihood Plug-in Predictor";
      risk_mleprt;

      "Loss Function Plug-in Predictor";
      risk_lfeprt;

      "======================================";
      "";
      "";

      alp_ind = alp_ind + 1;
   endo;

   h_ind = h_ind + 1;
endo;

end;
/****************************************************
** S U B R O U T I N E 
** Conditional on p
** compute asymptotic loss (Bias and Variance)
** for each h and the two predictors
*/
riskcalc:

RB_mle = zeros(h_n,1);
RB_mle2 = zeros(h_n,1);
RV_mle = zeros(h_n,1);

RB_lfe = zeros(h_n,1);
RV_lfe = zeros(h_n,1);

/* Create Companion Form matrices
*/
y_np = y_n*p_v;

if p_v == 1;   
    M = eye(y_n);
    F = Fdgp;
else;
    M = eye(y_n) | zeros(y_n*(p_v-1),y_n);
    F = ( Fdgp ~ zeros(y_n,y_n*(p_v-1)) )|
       ( eye(y_n*(p_v-1)) ~ zeros(y_n*(p_v-1),y_n)) ;
endif;

/* Create companion form A matrix
** Note: Adgp:       A_1, A_2, ...
**       A_mat: A_0, A_1, A_2, ...
*/
A_mat = zeros(y_np,y_np*(A_n+1));
A_ind = 1;
do until A_ind > A_n;
   A_mat[.,1+A_ind*y_np:(A_ind+1)*y_np] = M*Adgp[.,1+(A_ind-1)*y_n:A_ind*y_n]*M';
   A_ind = A_ind + 1;
endo;


/* Compute covariances Y,Z
** order 0,1,...h_max
*/
GAMzy = zeros(y_np,y_np*(h_max+1));
h_ind = 0;
do until h_ind > h_max;

   j_ind = h_ind;
   do until j_ind > A_n;
      GAMzy[.,1+y_np*h_ind:y_np*(h_ind+1)] 
        = GAMzy[.,1+y_np*h_ind:y_np*(h_ind+1)] 
        + A_mat[.,1+y_np*j_ind:y_np*(j_ind+1)] * (M*SIGeps*M') * PowerOf(F,j_ind-h_ind)';
      j_ind = j_ind + 1;
   endo;
   
   h_ind = h_ind + 1;
   
endo;

GAMyy  = reshape( inv( eye((y_np)^2) - (F.*.F))*vec(M*SIGeps*M') , y_np, y_np) ;

h_ind = 1;
do until h_ind > h_n;
   /* Begin loop: for each forecasting horizon
   */
   
   /* Compute asymptotic bias of estimators
   */
   mu_lfe = 0;
   mu_mle = 0;
   j_ind  = 0;
   do until j_ind > h[h_ind]-1;
      mu_lfe = mu_lfe + PowerOf(F,j_ind)
               * GAMzy[.,1+y_np*(h[h_ind]-j_ind):y_np*(h[h_ind]-j_ind+1)]*inv(GAMyy);
      mu_mle = mu_mle + PowerOf(F,h[h_ind]-1-j_ind)
               * GAMzy[.,1+y_np:y_np*2]*inv(GAMyy)
               * PowerOf(F,j_ind);
      j_ind = j_ind + 1;
   endo;
   
   /* Compute covariance matrix of estimators
   */
   V_mle = 0;
   V_lfe = 0;
   
   i_ind = 0;
   do until i_ind > h[h_ind]-1;
      j_ind = 0;
      do until j_ind > h[h_ind]-1;
         F_i   = PowerOf(F,i_ind);
         F_j   = PowerOf(F,j_ind);
         F_h1i = PowerOf(F,h[h_ind]-1-i_ind);
         F_h1j = PowerOf(F,h[h_ind]-1-j_ind);
         
         if i_ind GE j_ind;
            GAMstar = inv(GAMyy)*PowerOf(F,i_ind-j_ind);
         else;
            GAMstar = PowerOf(F,j_ind-i_ind)'*inv(GAMyy);
         endif;
         
         V_mle = V_mle + ( (M'*F_i*(M*SIGeps*M')*F_j'*M).*.(F_h1i'*inv(GAMyy)*F_h1j) );
         V_lfe = V_lfe + ( (M'*F_i*(M*SIGeps*M')*F_j'*M).*.GAMstar );
      
         j_ind = j_ind + 1;
      endo;
      i_ind = i_ind + 1;
   endo;

   /* Compute variance component of risk
   */
   RV_mle[h_ind] = sumc( diag( (Wloss.*.GAMyy) * V_mle ));
   RV_lfe[h_ind] = sumc( diag( (Wloss.*.GAMyy) * V_lfe ));

   /* Compute bias component of risk
   */
   
   B1_mle = sumc( diag( Wloss*M'*(mu_mle*GAMyy*mu_mle')*M ) );
   B2_mle = 0;
   B3_mle = 0;
   
   B1_lfe = sumc( diag( Wloss*M'*(mu_lfe*GAMyy*mu_lfe')*M ) );
   B2_lfe = 0;
   B3_lfe = 0;
   
   i_ind = 0;
   do until i_ind > h[h_ind]-1;
      j_ind = 0;
      do until j_ind > h[h_ind]-1;
      
         F_j = PowerOf(F,j_ind);
         F_i = PowerOf(F,i_ind);
         
         GAMstar = 0;
         l_ind = 0;
         do until maxc( (h[h_ind]-i_ind+l_ind)|(h[h_ind]-j_ind+l_ind)) > A_n;
            GAMstar = GAMstar
                       + A_mat[.,1+y_np*(h[h_ind]-i_ind+l_ind):y_np*(h[h_ind]-i_ind+l_ind + 1)] 
                       * (M*SIGeps*M') 
                       * A_mat[.,1+y_np*(h[h_ind]-j_ind+l_ind):y_np*(h[h_ind]-j_ind+l_ind + 1)]';
            l_ind = l_ind + 1;
         endo; 
               
         B2_mle = B2_mle 
                + sumc( diag( Wloss*M'*F_i*GAMstar*F_j'*M ) );
         B2_lfe = B2_lfe 
                + sumc( diag( Wloss*M'*F_i*GAMstar*F_j'*M ) );

         j_ind = j_ind + 1;
      endo;   
      i_ind = i_ind + 1;
   endo;   

   i_ind = 0;
   do until i_ind > h[h_ind]-1;

      F_i = PowerOf(F,i_ind);
      GAMzyhi = GAMzy[.,1+y_np*(h[h_ind]-i_ind):y_np*(h[h_ind]-i_ind+1)];

      B3_mle = B3_mle
               - 2*sumc( diag( Wloss*M'*(F_i*GAMzyhi*mu_mle')*M ) );

      B3_lfe = B3_lfe
               - 2*sumc( diag( Wloss*M'*(F_i*GAMzyhi*mu_lfe')*M ) );

      i_ind = i_ind + 1;

   endo;

   RB_mle[h_ind] = B1_mle + B2_mle + B3_mle;
   RB_lfe[h_ind] = B1_lfe + B2_lfe + B3_lfe;

   RB_mle2[h_ind] = RB_lfe[h_ind] + sumc(diag(Wloss*M'*(mu_mle-mu_lfe)*GAMyy*(mu_mle-mu_lfe)'*M));

   h_ind = h_ind + 1;
   
   /* End loop: for each forecasting horizon
   */
endo;

return;
/****************************************
** E N D   O F   S U B R O U T I N E
****************************************/

proc (1) = PowerOf(A,h);
local i,Ah;

Ah = eye(rows(A));

if h > 0;
   i = 1;
   do until i > h;
      Ah = Ah*A;
      i = i+1;
   endo;
endif;

retp(Ah);
endp;
